dnl -*- m4 -*-
dnl
dnl  This file is part of WEEVIL.
dnl
dnl  Authors: See the file AUTHORS for full details.
dnl  
dnl  Copyright (C) 2007 Università della Svizzera Italiana
dnl  Weevil is free software: you can redistribute it and/or modify
dnl  it under the terms of the GNU General Public License as published by
dnl  the Free Software Foundation, either version 3 of the License, or
dnl  (at your option) any later version.
dnl
dnl  Weevil is distributed in the hope that it will be useful,
dnl  but WITHOUT ANY WARRANTY; without even the implied warranty of
dnl  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
dnl  GNU General Public License for more details.
dnl
dnl  You should have received a copy of the GNU General Public License
dnl  along with Weevil.  If not, see <http://www.gnu.org/licenses/>.
dnl
dnl
dnl ===================== Preliminary definitions ======================
dnl --- define the number of siena components: we use 6 Siena servers
define(`SienaCompNumber', `6')dnl
dnl --- using the component number we define a range from 1 to 6
define(`SienaCompRange', `WVL_SYS_Range(`', 1, SienaCompNumber)')dnl
dnl --- for each element in range we create the list of Siena components: S1 through S6
define(`SienaComps', `WVL_SYS_Range(`S', 1, SienaCompNumber)')dnl
dnl --- we use the same range to define a client for each Siena server, clients range from D1 to D6
define(`SienaActors', `WVL_SYS_Range(`D', 1, SienaCompNumber)')dnl
dnl --- we can also use an actor to shape the traffic between components. Here we don't
dnl define(`TrafficControlActors', `WVL_SYS_Range(`DTC', 1, SienaCompNumber)')dnl
dnl --- The workload uses its own specific identifiers for actors: they go from C1 to C6
define(`SienaWkldActors', `WVL_SYS_Range(`C', 1, SienaCompNumber)')dnl
dnl --- We define SienaMaps as a collection of mapping relations (from CHMap1 to CHMap6), 
dnl --- each relation is defined later in the configuration file
define(`SienaMaps', `WVL_SYS_Range(`CHMap', 1, SienaCompNumber)')dnl
dnl
dnl ===================== Experiment ======================
dnl We define experiment "Exp0" using the following macros: 
dnl --- MyWorkload: definition of the actual workload file together with the list of included workload actors (C1 to C6)
dnl --- `SienaActors'`,LL': expands to the list of experiment actors (D1 to D6) plus "LL", an actor we use to dynamically configure network topology
dnl --- Bed0: is the testbed, a list of macros expanding to actual hosts and defined below
dnl --- Siena: expands to the SUE (System Under Experimentation) defining components, start order, and relations
dnl --- SienaMaps: the list of mappings defined above
dnl Then we declare that we want the experiment to clean up the testbed when finished (Y), we allow a maximum of 10 parallel remote invocations, and set
dnl the maximum experiment timeout to 600 seconds. Please refer to the Weevil manual for the complete syntax of the commands, below is the macro signature.
dnl SIGNATURE: WVL_SYS_Experiment(`id', `workload', `actors', `testbed', `SUE', `componentHostsMapping', `clean workspace', `parallel', `monitor', `timeout(sec)')dnl
WVL_SYS_Experiment(`Exp0', `MyWorkload', `SienaActors'`, LL', `Bed0', `Siena', `SienaMaps',`Y',`10',,`600')dnl
dnl
dnl ===================== Workload ======================
dnl
dnl The workload file is shortWorkload.wkld with actors C1 to C6
WVL_SYS_Workload(`MyWorkload', `shortWorkload.wkld', `SienaWkldActors')dnl
dnl
dnl ===================== Testbed Conceptual Model Configuration ====================
dnl
dnl -- TestBed --
dnl
dnl The testbed is composed of two hosts H0 and VHO
dnl SIGNATURE: WVL_SYS_Testbed( ID, Host List)dnl
WVL_SYS_Testbed( `Bed0', `H0, VH0')dnl
dnl
dnl -- Host --
dnl
dnl Host definitions: with id, IP address, username, shell path, Java path, experiment run directory, host type
dnl SIGNATURE: WVL_SYS_Host(`id', `address', `account', `shell', `java', `weevilroot (workspace)', `hosttype')dnl
WVL_SYS_Host(`H0', `192.168.68.204', `tof', `/bin/sh', `/opt/java/bin/java', `/home/tof/temp/weevil-tests/demo-run', `Linux')dnl
WVL_SYS_Host(`VH0', `research.inf.unisi.ch', `tof', `/bin/sh', `/usr/bin/java', `/home/tof/weevil-run', `Linux')dnl
dnl
dnl -- HostType --
dnl
WVL_SYS_HostType(`Linux')dnl
dnl
dnl -- HostProp --
dnl
dnl
dnl ======================= SUE Conceptual Model Configuration =======================
dnl
dnl -- SUE --
dnl
dnl The "Siena" SUE has components defined in SienaComps (S1 to S6), no relations are used, starting order for components
dnl is defined with other two macros
dnl SIGNATURE: WVL_SYS_SUE( `id', `components', `relations', `order', `prestart')
WVL_SYS_SUE( `Siena', `SienaComps', `', `Order', `PreStartComponents')dnl
dnl
dnl -- ComponentType --
dnl
dnl Here we define the experiment component types:
dnl --- we have SienaDVDRPServer with start and stop scripts, no start arguments nor config, and two log files
dnl SIGNATURE: WVL_SYS_ComponentType(`id', `startScript', `stopScript', `startArgs', `config', `logs')
WVL_SYS_ComponentType(`SienaDVDRPServer', `SienaDVDRPServer_startScript', `SienaDVDRPServer_stopScript', `', `', dnl
`WVL_SYS_Echo(WVL_Experiment_Name)`_'WVL_Component_ID.log, dnl
WVL_SYS_Echo(WVL_Experiment_Name)`_'WVL_Component_ID.err')dnl
dnl
dnl - SienaServer Start/Stop Script -
dnl
dnl ----- Here we prepare the start script  using properties and relationships set for each component
define(`SienaDVDRPServer_startScript', dnl
`WVL_SYS_Echo(`WVL_Host_'WVL_SYS_Echo(theHost)`_java') -classpath dnl
theSoftwarePath/siena-plastic.jar dnl
dnl :theSoftwarePath/B3GNetworkModel.jar:dnl
dnl theSoftwarePath/MultiRadioNetworking.jar:theSoftwarePath/PlasticAddress.jar:dnl
dnl theSoftwarePath/PlasticUtil.jar dnl
siena.StartDVDRPServer -id WVL_Component_ID -WVL_SYS_Echo(`WVL_Component_'WVL_Component_ID`_Protocol') \
-port WVL_SYS_Echo(`WVL_Component_'WVL_Component_ID`_Port') -host WVL_SYS_Echo(`WVL_Host_'theHost`_address') -logSeverity DEBUG \
 -log theCompPath/WVL_SYS_Echo(WVL_Experiment_Name)`_'WVL_Component_ID.log -err theCompPath/WVL_SYS_Echo(WVL_Experiment_Name)`_'WVL_Component_ID.err &')dnl
dnl
define(`SienaDVDRPServer_stopScript', dnl
`read pid <theCompPath/WVL_SYS_Echo(WVL_Experiment_Name)`_'WVL_Component_ID.pid
echo $pid
kill $pid')dnl
dnl
dnl -- Component --
dnl
dnl For each element in SienaCompRange (S1 to S6) we instantiate a component of type SienaDVDRPServer
WVL_SYS_Foreach(`i', `WVL_SYS_Component(`S'i, `SienaDVDRPServer')', SienaCompRange)dnl
dnl
dnl -- ComponentProp --
dnl
dnl For each instantiated component we assign listening ports and communication protocol
WVL_SYS_Foreach(`i', `WVL_SYS_ComponentProp(`S'i, `Port', `500'i)', SienaCompRange)dnl
WVL_SYS_Foreach(`i', `WVL_SYS_ComponentProp(`S'i, `Protocol', `tcp')', SienaCompRange)dnl
dnl
dnl -- ComponentRelation --
dnl
dnl
dnl -- ComponentOrder --
dnl
dnl Here we define the start order for our components 
WVL_SYS_ComponentOrder(`Order', `S1, 0, S2, 0, S3, 0, S4, 0, S5, 0 , S6, 10')dnl
dnl -----------------------Pre Start Component-----------------------------
dnl
dnl All components need to be started before the experiment is run
WVL_SYS_ComponentPreStart(`PreStartComponents', `SienaComps')dnl
dnl
dnl ===================== Mappings(SUE-Testbed) ====================
dnl
dnl -- ComponentHost --
dnl
dnl Here we use some macro to split the components between H0 and VH0 (S1 to S3 run on H0, S4 to S6 on VH0)
define(`halfCompRange', `WVL_SYS_Range(`', 1, `eval(SienaCompNumber / 2)')')dnl
define(`restCompRange', `WVL_SYS_Range(`', `eval(SienaCompNumber / 2 + 1)', SienaCompNumber)')dnl
WVL_SYS_Foreach(`i', `WVL_SYS_ComponentHost(`CHMap'i, `S'i, `H0')', halfCompRange)dnl
WVL_SYS_Foreach(`i', `WVL_SYS_ComponentHost(`CHMap'i, `S'i, `VH0')', restCompRange)dnl
dnl
dnl -- ComponentHostType --
dnl
dnl Here we define the files needed for SienaDVDRPServer on hosts of type Linux. 
dnl (The content of the binaries directory will be copied to the testbed hosts)
WVL_SYS_ComponentHostType( `SienaDVDRPServer', `Linux', `binaries')dnl
dnl
dnl ===================== Mappings(SUE-Actor) ====================
dnl
dnl -- ActorProgram to trigger subscriptions, unsubscriptions and publications --
dnl
dnl The actors executing the workload are written in Java, classes are found in directory "actor", the class to be invoked is
dnl siena.SienaActorThinClient, with arguments SienaActor_args and classpath SienaActor_classpath
WVL_SYS_ActorProgram(`ClientProgram', `Java', `./actor', `siena.SienaActorThinClient', `SienaActor_args', `SienaActor_classpath')dnl
dnl
dnl Arguments for workload actors are: communication protocol, server address, server port, client port, client ID
define(`SienaActor_args', `WVL_SYS_Echo(`WVL_Component_'theComponent`_Protocol') dnl
WVL_SYS_Echo(`WVL_Host_'theHost`_address') WVL_SYS_Echo(`WVL_Component_'theComponent`_Port') dnl
WVL_SYS_Echo(`WVL_Actor_'WVL_Actor_ID`_Port') WVL_SYS_Echo(`WVL_Actor_ID')')dnl
dnl
dnl Classpath for workload actors includes jar siena-plastic.jar situated in the running directory
define(`SienaActor_classpath', `theSoftwarePath/siena-plastic.jar')dnl
dnl
dnl -- ActorProgram to dynamically link servers
WVL_SYS_ActorProgram(`LinkProgram', `Java', `./actor', `siena.LinkActor', `LinkActor_args', `LinkActor_classpath')dnl
dnl -- ActorProgram to for traffic control and shaping
WVL_SYS_ActorProgram(`TCProgram', `Shell', `./actor', `trafficControlActor.sh', `', `', `WVL_Experiment_Name`_'WVL_Actor_ID`.log'')dnl
dnl
define(`LinkActor_args', WVL_SYS_Foreach(`i', computeLinkArgs(i, `WVL_SYS_Echo(`WVL_ComponentHost_'i`_host')'), WVL_SUE_Siena_components))dnl
define(`computeLinkArgs', `WVL_SYS_Echo($1) WVL_SYS_Echo(`WVL_Component_'$1`_Protocol') dnl
`WVL_SYS_Echo(`WVL_Host_'$2`_address')' WVL_SYS_Echo(`WVL_Component_'$1`_Port') ')dnl
dnl 
dnl
define(`LinkActor_classpath', `theSoftwarePath/siena-plastic.jar')dnl
dnl
dnl -- Actor --
dnl
dnl Here we map each client instance (D*) with its workload actor (C*), actor program, and component (S*)
dnl SIGNATURE: WVL_SYS_Actor(id, workloadProcess id, actorProgram, component, dir, props)
WVL_SYS_Foreach(`i', `WVL_SYS_Actor(`D'i, `C'i, `ClientProgram', `S'i)', SienaCompRange)dnl
dnl WVL_SYS_Foreach(`i', `WVL_SYS_Actor(`DTC'i, `TC'i, `TCProgram', `S'i)', SienaCompRange)dnl
dnl In addition we only have on actor to dynamically configure the network topology
WVL_SYS_Actor(`LL', `CL', `LinkProgram', `S1')dnl
dnl
dnl
dnl -- ActorProp --
dnl
dnl Here we set the listening port for each client
WVL_SYS_Foreach(`i', `WVL_SYS_ActorProp(`D'i, `Port', `600'i)', SienaCompRange)dnl
dnl

