//
// -*- C++ -*-
//
//  This file is part of WEEVILGEN.
//
//  Authors: See the file AUTHORS for full details.
//
//  Copyright (C) 2002-2004 University of Colorado
//  Weevil is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  Weevil is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with Weevil.  If not, see <http://www.gnu.org/licenses/>.
//
//
string Message_Generator::generator(string eventtype) {
  if (eventtype.compare("SUB") == 0){
  	set<string> attrs;
    int acount = rand_normal_range(m_constr_min, m_constr_max);

	while(attrs.size() < acount)
	attrs.insert(attr_dist.rand_pick());

	set<string>::const_iterator vi;
	ostringstream str_stream;

	for(vi = attrs.begin(); vi != attrs.end(); ++vi) {
	  char t = types_dist.rand_pick();
	  if (vi != attrs.begin())
		str_stream << " & ";
	  switch(t){
		case 's':{
			string op_str, str_str;
			op_str = op_string_dist.rand_pick();
			str_str = str_dist.rand_pick();
			str_stream << "s " << *vi << ' ' << op_str << ' ' << str_str;
		    break;
		}
		case 'i':{
			string op_str = op_dist.rand_pick();
			int int_value = int_dist.rand_pick();
	    	str_stream << "i " << *vi << ' ' << op_str << ' ' << int_value;
		    break;
		}
		case 'b':{
			string op_str = op_dist.rand_pick();
			bool bool_value = bool_dist.rand_pick();
			str_stream << "b " << *vi << ' ' << op_str << ' ' << bool_value;
		    break;
		}
		default: str_stream << "???     ";
			break;
	  }
	}
    return str_stream.str();
  }
  else if (eventtype.compare("PUB") == 0){
	set<string> attrs;
 	int acount = rand_normal_range(m_attr_min, m_attr_max);

	while(attrs.size() < acount)
	  attrs.insert(attr_dist.rand_pick());

	set<std::string>::const_iterator vi;
	ostringstream str_stream;

	for(vi = attrs.begin(); vi != attrs.end(); ++vi) {
 	  char t = types_dist.rand_pick();
	  if (vi != attrs.begin())
  		str_stream << " & ";
      switch(t) {
      	case 's':
	    	str_stream << "s " << *vi << ' ' << str_dist.rand_pick();
          	break;
		case 'i':
			str_stream << "i " << *vi << ' ' << int_dist.rand_pick();
   			break;
        case 'b':
	    	str_stream << "b " << *vi << ' ' << bool_dist.rand_pick();
            break;
        default:  str_stream << "???     "; break;
        }
      }
      return str_stream.str();
	}
	else if (eventtype.compare("UNSUB") == 0){
		ostringstream str_stream;
		str_stream << "";
		return str_stream.str();
	}
	return 0;
}

int Message_Generator::readdist(){
  //
  // for attr
  //
  ifstream attr_dist_s(m_attr_dist_f);
  if (!attr_dist_s) {
    cerr << "Error: Could not open attribute distribution file: "
    << m_attr_dist_f << endl;
    return 1;
  }
  attr_dist.read_from(attr_dist_s);
  attr_dist_s.close();

  //
  // for constr
  //
  ifstream constr_dist_s(m_constr_dist_f);
  if (!constr_dist_s) {
  	cerr << "Error: Could not open constrict distribution file: "
   	<< m_constr_dist_f << endl;
    return 1;
  }
  constr_dist.read_from(constr_dist_s);
  constr_dist_s.close();

  //
  // read distribution file for strings
  //
  ifstream str_dist_s(m_str_dist_f);
  if (!str_dist_s) {
	cerr << "error: could not open string values distribution file: "
	<< m_str_dist_f << endl;
	return 1;
  }
  str_dist.read_from(str_dist_s);
  str_dist_s.close();

  //
  // read distribution file for integers
  //
  ifstream int_dist_s(m_int_dist_f);
  if (!int_dist_s) {
	cerr << "error: could not open integer values distribution file: "
	<< m_int_dist_f << endl;
	return 1;
  }
  int_dist.read_from(int_dist_s);
  int_dist_s.close();

  //
  // read distribution file for booleans
  //
  ifstream bool_dist_s(m_bool_dist_f);
  if (!bool_dist_s) {
	cerr << "error: could not open booleans values distribution file: "
	<< m_bool_dist_f << endl;
	return 1;
  }
  bool_dist.read_from(bool_dist_s);
  bool_dist_s.close();

  //
  // read distribution file for types
  //
  ifstream types_dist_s(m_types_dist_f);
  if (!types_dist_s) {
	cerr << "error: could not open types distribution file: "
	<< m_types_dist_f << endl;
	return 1;
  }
  types_dist.read_from(types_dist_s);
  types_dist_s.close();

  //
  // read distribution file for operators
  //
  ifstream op_dist_s(m_op_dist_f);
  if (!op_dist_s) {
	cerr << "error: could not open operators distribution file: "
	<< m_op_dist_f << endl;
	return 1;
  }
  op_dist.read_from(op_dist_s);
  op_dist_s.close();

  //
  // for string operators
  //
  ifstream op_string_dist_s(m_op_string_dist_f);
  if (!op_string_dist_s) {
  	cerr << "Error: Could not open string operators distribution file: "
   	<< m_op_string_dist_f << endl;
    return 1;
  }
  op_string_dist.read_from(op_string_dist_s);
  op_string_dist_s.close();
}

/**
 *	random value with uniform distribution in [0,max)
 */
int Message_Generator::rand_normal(int max) {
    return (int) (1.0*max*rand()/(RAND_MAX+1.0));
}

/**
 *	random value with uniform distribution in [min,max)
 */
int Message_Generator::rand_normal_range(int min, int max) {
    return min + rand_normal(max - min);
}

