// -*- C++ -*-
//
//  This file is part of WEEVILGEN.
//
//  Authors: See the file AUTHORS for full details.
//
//  Copyright (C) 2002-2004 University of Colorado
//  Weevil is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  Weevil is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with Weevil.  If not, see <http://www.gnu.org/licenses/>.
//
//
#ifndef WSET_H
#define WSET_H

#include <stdlib.h>
#include <map>
#include <vector>
#include <list>

using namespace std;

typedef unsigned int weight_t;

typedef list<string> filter_container;
typedef vector<filter_container> if_container;

/** "weighted" set
 *
 *  represents a set of elements, each having a "weight" parameter
 **/
template <class T>
class wset {
private:
    map<weight_t, T>	wmap;
    weight_t		weight_size;
    static weight_t rand_normal(weight_t max) {
	return (weight_t) (1.0*max*rand()/(RAND_MAX+1.0));
    }

 public:
    void clear() {
	wmap.clear();
	weight_size = 0;
    }

    void add(weight_t w, const T & v) {
	if (w > 0) {
	    weight_size += w;
	    wmap.insert(wmap.end(), 
			typename map<weight_t, T>::value_type(weight_size - 1, v));        
	}
    }

    T rand_pick() const {
	return (*wmap.lower_bound(rand_normal(weight_size))).second;
    }

    bool read_from(istream & in) {
	clear();
	weight_t w;
	T v;
	while (in >> w >> v) 
	    add(w, v);
	return true;
    }
};
#endif
