# -*- sh -*-
#
#  This file is part of WEEVIL.
#
#  Authors: See the file AUTHORS for full details.
#
#  Copyright (C) 2002-2006 University of Colorado
#
#  Weevil is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  Weevil is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with Weevil.  If not, see <http://www.gnu.org/licenses/>.
#
#
#
# this script is called by the experiment master script as the per-actor-workload
# intepreter.
#
# check the arguments
#
if [ $# -ne 6 ]
then
	echo "parameter error: You have $# arugment(s) instead of 6."
	echo "usage: sh weevil-sh-actor.sh <initial delay> <exproot> <actorpath> <workload> <userprogram name> <userprogram config>"
	exit
elif [ ! -f $3/$4 ]
then
	echo "parameter error: The workload file you refer in the 4th argument does not exist."
	echo "usage: sh weevil-sh-actor.sh <initial delay> <exproot> <actorpath> <workload> <userprogram name> <userprogram config>"
	exit
fi
#
# waiting for the start time
#
echo $$ >>$3/pids
sleep `echo $1 / 1000 | bc -l` &
tmppid=$!
echo $tmppid >>$3/pids
wait $tmppid
#
# record experiment start time
#
expstartsecs=`date +%s`
echo $expstartsecs > $2/wvl_expstart.time
echo "start time $expstartsecs"
#
# decide the line number of the workload file
#
linenum=`cat $3/$4 | wc -l`
enterornot=`tail -n 1 $3/$4|wc -l`
if [ $enterornot -eq 0 ]
then
    linenum=`expr $linenum + 1`
else
    linenum=`expr $linenum`
fi
#
# reading the per-client workload file
#
i=$linenum
while [ $i -gt 0 ]
do
    line=`tail -n $i $3/$4 | head -n 1`
    msg=`echo $line|awk 'BEGIN {FS=")"}; {print $1")"}'`
    command=`echo $line|awk -F[\(] '{print $1}'`
    parameter=`echo $msg|awk -F[\(\)] '{print $2}'`    
    condition=`echo $line|awk 'BEGIN {FS="),"}; {print $2}'|awk -F[,] '{print $1}'`
    variable=`echo $line|awk 'BEGIN {FS="),"}; {print $2}'|awk -F[,] '{print $2}'`
    scaledParameter=`echo $parameter / 1000 | bc -l`    

    case $command in
	"WVL_Wait") sleep $scaledParameter &
		    tmppid=$!		    
		    echo $tmppid >>$3/pids		    
		    wait $tmppid;;
	*) sh $5 $6 "$msg" "$condition" "$variable" &
		     tmppid=$!
		     echo $tmppid >>$3/pids
		     wait $tmppid;;
    esac
    i=`expr $i - 1`
done
echo "done"
